@@ -1,8 +1,7 @@
 from dateutil.tz import tzlocal
 import stream
 import time
-from stream.exceptions import ApiKeyException, InputException,\
-    CustomFieldException
+from stream.exceptions import ApiKeyException, InputException
 import random
 from unittest.case import TestCase
 
@@ -19,6 +18,7 @@ def connect_debug():
 
 client = connect_debug()
 user1 = client.feed('user:1')
+user2 = client.feed('user:2')
 aggregated2 = client.feed('aggregated:2')
 aggregated3 = client.feed('aggregated:3')
 topic1 = client.feed('topic:1')
@@ -31,6 +31,7 @@ class ClientTest(TestCase):
         # DEBUG account details
         self.c = client
         self.user1 = user1
+        self.user2 = user2
         self.aggregated2 = aggregated2
         self.aggregated3 = aggregated3
         self.topic1 = topic1
@@ -43,6 +44,14 @@ class ClientTest(TestCase):
         self.assertEqual(client.api_key, 'thierry')
         self.assertEqual(client.api_secret, 'pass')
         self.assertEqual(client.site_id, '1')
+        
+    def test_heroku_real(self):
+        url = 'https://bvt88g4kvc63:twc5ywfste5bm2ngqkzs7ukxk3pn96yweghjrxcmcrarnt3j4dqj3tucbhym5wfd@getstream.io/?site=669'
+        os.environ['STREAM_URL'] = url
+        client = stream.connect()
+        self.assertEqual(client.api_key, 'bvt88g4kvc63')
+        self.assertEqual(client.api_secret, 'twc5ywfste5bm2ngqkzs7ukxk3pn96yweghjrxcmcrarnt3j4dqj3tucbhym5wfd')
+        self.assertEqual(client.site_id, '669')
 
     def test_heroku_overwrite(self):
         url = 'https://thierry:pass@getstream.io/?site=1'
@@ -53,13 +62,14 @@ class ClientTest(TestCase):
         self.assertEqual(client.site_id, 'c')
 
     def test_token_retrieval(self):
-        token = self.user1.token
+        self.user1.token
 
     def test_add_activity(self):
+        feed = client.feed('user:py1')
         activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
-        response = self.user1.add_activity(activity_data)
+        response = feed.add_activity(activity_data)
         activity_id = response['id']
-        activities = self.user1.get(limit=1)['results']
+        activities = feed.get(limit=1)['results']
         self.assertEqual(activities[0]['id'], activity_id)
 
     def test_add_activity_to(self):
@@ -124,32 +134,36 @@ class ClientTest(TestCase):
         self.assertEqual(get_activity_ids, activity_ids[::-1])
 
     def test_follow(self):
+        feed = client.feed('user:test_follow')
+        agg_feed = client.feed('aggregated:test_follow')
         actor_id = random.randint(10, 100000)
         activity_data = {'actor': actor_id, 'verb': 'tweet', 'object': 1}
-        activity_id = self.user1.add_activity(activity_data)['id']
-        self.aggregated2.follow('user:1')
+        activity_id = feed.add_activity(activity_data)['id']
+        agg_feed.follow('user:test_follow')
         time.sleep(10)
-        activities = self.aggregated2.get(limit=3)['results']
+        activities = agg_feed.get(limit=3)['results']
         activity = self._get_first_aggregated_activity(activities)
         activity_id_found = activity['id'] if activity is not None else None
         self.assertEqual(activity_id_found, activity_id)
 
     def test_follow_private(self):
         feed = client.feed('secret:py1')
+        agg_feed = client.feed('aggregated:test_follow_private')
         actor_id = random.randint(10, 100000)
         activity_data = {'actor': actor_id, 'verb': 'tweet', 'object': 1}
         activity_id = feed.add_activity(activity_data)['id']
-        self.aggregated2.follow('secret:py1')
+        agg_feed.follow('secret:py1')
         time.sleep(10)
-        activities = self.aggregated2.get(limit=3)['results']
+        activities = agg_feed.get(limit=3)['results']
         activity = self._get_first_aggregated_activity(activities)
         activity_id_found = activity['id'] if activity is not None else None
         self.assertEqual(activity_id_found, activity_id)
 
     def test_flat_follow(self):
+        feed = client.feed('user:test_flat_follow')
         activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
-        activity_id = self.user1.add_activity(activity_data)['id']
-        self.flat3.follow('user:1')
+        activity_id = feed.add_activity(activity_data)['id']
+        self.flat3.follow('user:test_flat_follow')
         time.sleep(10)
         activities = self.flat3.get(limit=3)['results']
         activity = self._get_first_activity(activities)
@@ -200,7 +214,7 @@ class ClientTest(TestCase):
 
     def test_empty_followers(self):
         asocial = client.feed('user:asocialpython')
-        followers = asocial.following()
+        followers = asocial.followers()
         self.assertEqual(len(followers['results']), 0)
         self.assertEqual(followers['results'], [])
 
@@ -254,6 +268,42 @@ class ClientTest(TestCase):
         activities = self.user1.get(limit=2, id_lt=activity_id_two)['results']
         self.assertEqual(activities[0]['id'], activity_id)
 
+    def test_mark_read(self):
+        notification_feed = client.feed('notification:py3')
+        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
+        notification_feed.add_activity(activity_data)
+        activity_data = {'actor': 2, 'verb': 'add', 'object': 2}
+        notification_feed.add_activity(activity_data)
+        activity_data = {'actor': 3, 'verb': 'watch', 'object': 2}
+        notification_feed.add_activity(activity_data)
+        activities = notification_feed.get()['results']
+        for activity in activities:
+            self.assertFalse(activity['is_read'])
+        activities = notification_feed.get(mark_read=True)['results']
+        activities = notification_feed.get()['results']
+        for activity in activities:
+            self.assertTrue(activity['is_read'])
+
+    def test_mark_read_by_id(self):
+        notification_feed = client.feed('notification:py2')
+        activity_data = {'actor': 1, 'verb': 'tweet', 'object': 1}
+        notification_feed.add_activity(activity_data)['id']
+        activity_data = {'actor': 2, 'verb': 'add', 'object': 2}
+        notification_feed.add_activity(activity_data)['id']
+        activity_data = {'actor': 3, 'verb': 'watch', 'object': 2}
+        notification_feed.add_activity(activity_data)['id']
+        activities = notification_feed.get()['results']
+        ids = []
+        for activity in activities:
+            ids.append(activity['id'])
+            self.assertFalse(activity['is_read'])
+        ids = ids[:2]
+        notification_feed.get(mark_read=ids)
+        activities = notification_feed.get()['results']
+        for activity in activities:
+            if activity['id'] in ids:
+                self.assertTrue(activity['is_read'])
+
     def test_api_key_exception(self):
         self.c = stream.connect(
             '5crf3bhfzesnMISSING',
@@ -336,6 +386,32 @@ class ClientTest(TestCase):
         self.assertEqual(activities[0]['foreign_id'], 'tweet:11')
         self.assertDatetimeAlmostEqual(activities[0]['time'], utcnow)
         self.assertNotEqual(activities[1]['foreign_id'], 'tweet:11')
+        
+    def test_time_ordering(self):
+        '''
+        datetime.datetime.utcnow() is our recommended approach
+        so if we add an activity
+        add one using time
+        add another activity it should be in the right spot
+        '''
+        now = datetime.datetime.utcnow
+        feed = self.user2
+        for index, activity_time in enumerate([None, now, None]):
+            time.sleep(1)
+            if activity_time is not None:
+                activity_time = activity_time()
+                middle = activity_time
+            activity_data = {'actor': 1, 'verb': 'tweet',
+                             'object': 1, 'foreign_id': 'tweet:%s' % index, 'time': activity_time}
+            feed.add_activity(activity_data)
+
+        activities = feed.get(limit=3)['results']
+        # the second post should have overwritten the first one (because they
+        # had same id)
+        self.assertEqual(activities[0]['foreign_id'], 'tweet:2')
+        self.assertEqual(activities[1]['foreign_id'], 'tweet:1')
+        self.assertEqual(activities[2]['foreign_id'], 'tweet:0')
+        self.assertDatetimeAlmostEqual(activities[1]['time'], middle)
 
     def test_missing_actor(self):
         activity_data = {'verb': 'tweet', 'object':
